@@ -17,6 +17,8 @@
 #' @param input \code{robyn_export()}'s output or \code{pareto_aggregated.csv} results.
 #' @param dep_var_type Character. For dep_var_type 'revenue', ROI is used for clustering.
 #' For conversion', CPA is used for clustering.
+#' @param cluster_by Character. Any of: "performance" or "hyperparameters".
+#' @param max_clusters Integer. Maximum number of clusters.
 #' @param limit Integer. Top N results per cluster. If kept in "auto", will select k
 #' as the cluster in which the WSS variance was less than 5\%.
 #' @param weights Vector, size 3. How much should each error weight?
@@ -37,11 +39,15 @@
 #' }
 #' @return List. Clustering results as labeled data.frames and plots.
 #' @export
-robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", limit = 1,
+robyn_clusters <- function(input, dep_var_type,
+                           cluster_by = "hyperparameters",
+                           all_media = NULL,
+                           k = "auto", wss_var = 0.06, max_clusters = 10, limit = 1,
                            weights = rep(1, 3), dim_red = "PCA",
                            quiet = FALSE, export = FALSE, seed = 123,
                            ...) {
   set.seed(seed)
+  check_opts(cluster_by, c("performance", "hyperparameters"))
   if ("robyn_outputs" %in% class(input)) {
     if (is.null(all_media)) {
       aux <- colnames(input$mediaVecCollect)
@@ -51,21 +57,18 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
       path <- paste0(getwd(), "/")
     }
     # Pareto and ROI data
-    xDecompAgg <- input$xDecompAgg
-    df <- .prepare_df(xDecompAgg, all_media, dep_var_type)
+    x <- xDecompAgg <- input$xDecompAgg
+    if (cluster_by %in% "hyperparameters") x <- input$resultHypParam
+    df <- .prepare_df(x, all_media, dep_var_type, cluster_by)
   } else {
-    if (all(c("solID", "mape", "nrmse", "decomp.rssd") %in% names(input)) && is.data.frame(input)) {
-      df <- .prepare_df(input, all_media, dep_var_type)
-    } else {
-      stop(paste(
-        "You must run robyn_outputs(..., clusters = TRUE) or",
-        "pass a valid data.frame (sames as pareto_aggregated.csv output)",
-        "in order to use robyn_clusters()"
-      ))
-    }
+    stop(paste(
+      "You must run robyn_outputs(..., clusters = TRUE) or",
+      "pass a valid data.frame (sames as pareto_aggregated.csv output)",
+      "in order to use robyn_clusters()"
+    ))
   }
 
-  ignore <- c("solID", "mape", "decomp.rssd", "nrmse", "nrmse_test", "nrmse_train", "pareto")
+  ignore <- c("solID", "mape", "decomp.rssd", "nrmse", "nrmse_test", "nrmse_train", "nrmse_val", "pareto")
 
   # Auto K selected by less than 5% WSS variance (convergence)
   min_clusters <- 3
@@ -73,10 +76,12 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
   if ("auto" %in% k) {
     cls <- tryCatch(
       {
-        clusterKmeans(df,
-          k = NULL, limit = limit_clusters, ignore = ignore,
-          dim_red = dim_red, quiet = TRUE, seed = seed
-        ) # , ...)
+        suppressMessages(
+          clusterKmeans(df,
+            k = NULL, limit = limit_clusters, ignore = ignore,
+            dim_red = dim_red, quiet = TRUE, seed = seed
+          )
+        )
       },
       error = function(err) {
         message(paste("Couldn't automatically create clusters:", err))
@@ -84,35 +89,50 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
       }
     )
     # if (is.null(cls)) return(NULL)
-    min_var <- 0.05
     k <- cls$nclusters %>%
       mutate(
         pareto = .data$wss / .data$wss[1],
         dif = lag(.data$pareto) - .data$pareto
       ) %>%
-      filter(.data$dif > min_var) %>%
+      filter(.data$dif > wss_var) %>%
       pull(.data$n) %>%
       max(., na.rm = TRUE)
-    if (k < min_clusters) k <- min_clusters
+    if (k < min_clusters) {
+      warning(sprintf("Too few clusters: %s. Setting to %s", k, min_clusters))
+      k <- min_clusters
+    }
     if (!quiet) {
       message(sprintf(
         ">> Auto selected k = %s (clusters) based on minimum WSS variance of %s%%",
-        k, min_var * 100
+        k, wss_var * 100
       ))
     }
+    if (k > max_clusters) {
+      warning(sprintf("Too many clusters: %s. Lowering to %s (max_clusters)", k, max_clusters))
+      k <- max_clusters
+    }
   }
 
   # Build clusters
   stopifnot(k %in% min_clusters:30)
-  cls <- clusterKmeans(df, k, limit = limit_clusters, ignore = ignore, dim_red = dim_red, quiet = TRUE) # , ...)
+  suppressMessages(
+    cls <- clusterKmeans(
+      df,
+      k = k, limit = limit_clusters, ignore = ignore,
+      dim_red = dim_red, quiet = TRUE, seed = seed
+    )
+  )
+  cls$df <- group_by(cls$df, .data$cluster) %>%
+    mutate(n = n()) %>%
+    ungroup()
 
   # Select top models by minimum (weighted) distance to zero
   all_paid <- setdiff(names(cls$df), c(ignore, "cluster"))
-  ts_validation <- ifelse(all(is.na(cls$df$nrmse_test)), FALSE, TRUE)
+  ts_validation <- ifelse("nrmse_test" %in% colnames(cls$df), TRUE, FALSE)
   top_sols <- .clusters_df(df = cls$df, all_paid, balance = weights, limit, ts_validation)
 
   # Build in-cluster CI with bootstrap
-  ci_list <- confidence_calcs(xDecompAgg, cls, all_paid, dep_var_type, k, ...)
+  ci_list <- confidence_calcs(xDecompAgg, cls, all_paid, dep_var_type, k, cluster_by, ...)
 
   output <- list(
     # Data and parameters
@@ -123,7 +143,7 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
     sim_n = ci_list$sim_n,
     errors_weights = weights,
     # Within Groups Sum of Squares Plot
-    wss = cls$nclusters_plot,
+    wss = cls$nclusters_plot + theme_lares(background = "white"),
     # Grouped correlations per cluster
     corrs = cls$correlations + labs(title = "Top Correlations by Cluster", subtitle = NULL),
     # Mean ROI per cluster
@@ -142,22 +162,24 @@ robyn_clusters <- function(input, dep_var_type, all_media = NULL, k = "auto", li
     write.csv(output$data, file = paste0(path, "pareto_clusters.csv"))
     write.csv(output$df_cluster_ci, file = paste0(path, "pareto_clusters_ci.csv"))
     ggsave(paste0(path, "pareto_clusters_wss.png"), plot = output$wss, dpi = 500, width = 5, height = 4)
-    db <- wrap_plots(
-      A = output$plot_clusters_ci,
-      B = output$plot_models_rois,
-      C = output$plot_models_errors,
-      design = "AA\nBC"
-    )
-    # Suppressing "Picking joint bandwidth of x" messages
-    suppressMessages(ggsave(paste0(path, "pareto_clusters_detail.png"),
-      plot = db, dpi = 500, width = 12, height = 14
-    ))
+    get_height <- ceiling(k / 2) / 6
+    db <- (output$plot_clusters_ci / (output$plot_models_rois + output$plot_models_errors)) +
+      patchwork::plot_layout(heights = c(get_height, 1), guides = "collect")
+    # Suppressing "Picking joint bandwidth of x" messages +
+    # In min(data$x, na.rm = TRUE) : no non-missing arguments to min; returning Inf warnings
+    # Setting try() to avoid error: One or both dimensions exceed the maximum (50000px).
+    #   Use `options(ragg.max_dim = ...)` to change the max
+    try(suppressMessages(suppressWarnings(ggsave(paste0(path, "pareto_clusters_detail.png"),
+      plot = db, dpi = 500, width = 12, height = 4 + length(all_paid) * 2, limitsize = FALSE
+    ))))
   }
 
   return(output)
 }
 
-confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n = 1000, sim_n = 10000, ...) {
+confidence_calcs <- function(
+    xDecompAgg, cls, all_paid, dep_var_type, k, cluster_by,
+    boot_n = 1000, sim_n = 10000, ...) {
   df_clusters_outcome <- xDecompAgg %>%
     filter(!is.na(.data$total_spend)) %>%
     left_join(y = dplyr::select(cls$df, c("solID", "cluster")), by = "solID") %>%
@@ -175,6 +197,9 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
     if (length(unique(df_outcome$solID)) < 3) {
       warning(paste("Cluster", j, "does not contain enough models to calculate CI"))
     } else {
+      if (cluster_by == "hyperparameters") {
+        all_paid <- unique(gsub(paste(paste0("_", HYPS_NAMES), collapse = "|"), "", all_paid))
+      }
       for (i in all_paid) {
         # Bootstrap CI
         if (dep_var_type == "conversion") {
@@ -186,9 +211,9 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
           v_samp <- df_chn$roi_total
         }
         boot_res <- .bootci(samp = v_samp, boot_n = boot_n)
-        boot_mean <- mean(boot_res$boot_means)
+        boot_mean <- mean(boot_res$boot_means, na.rm = TRUE)
         boot_se <- boot_res$se
-        ci_low <- ifelse(boot_res$ci[1] < 0, 0, boot_res$ci[1])
+        ci_low <- ifelse(boot_res$ci[1] <= 0, 0, boot_res$ci[1])
         ci_up <- boot_res$ci[2]
 
         # Collect loop results
@@ -206,7 +231,7 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
           rn = i,
           n = length(v_samp),
           boot_mean = boot_mean,
-          x_sim = rnorm(sim_n, mean = boot_mean, sd = boot_se)
+          x_sim = suppressWarnings(rnorm(sim_n, mean = boot_mean, sd = boot_se))
         ) %>%
           mutate(y_sim = dnorm(.data$x_sim, mean = boot_mean, sd = boot_se))
       }
@@ -217,6 +242,7 @@ confidence_calcs <- function(xDecompAgg, cls, all_paid, dep_var_type, k, boot_n
   sim_collect <- bind_rows(lapply(cluster_collect, function(x) {
     bind_rows(lapply(x$sim_collect, function(y) y))
   })) %>%
+    filter(.data$n > 0) %>%
     mutate(cluster_title = sprintf("Cl.%s (n=%s)", .data$cluster, .data$n)) %>%
     ungroup() %>%
     as_tibble()
@@ -284,36 +310,51 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
 }
 
 # ROIs data.frame for clustering (from xDecompAgg or pareto_aggregated.csv)
-.prepare_df <- function(x, all_media, dep_var_type) {
-  check_opts(all_media, unique(x$rn))
-
-  if (dep_var_type == "revenue") {
-    outcome <- select(x, .data$solID, .data$rn, .data$roi_total) %>%
-      tidyr::spread(key = .data$rn, value = .data$roi_total)
+.prepare_df <- function(x, all_media, dep_var_type, cluster_by) {
+  if (cluster_by == "performance") {
+    check_opts(all_media, unique(x$rn))
+    if (dep_var_type == "revenue") {
+      outcome <- select(x, .data$solID, .data$rn, .data$roi_total) %>%
+        tidyr::spread(key = .data$rn, value = .data$roi_total) %>%
+        removenacols(all = FALSE) %>%
+        select(any_of(c("solID", all_media)))
+    }
+    if (dep_var_type == "conversion") {
+      outcome <- select(x, .data$solID, .data$rn, .data$cpa_total) %>%
+        filter(is.finite(.data$cpa_total)) %>%
+        tidyr::spread(key = .data$rn, value = .data$cpa_total) %>%
+        removenacols(all = FALSE) %>%
+        select(any_of(c("solID", all_media)))
+    }
+    errors <- distinct(
+      x, .data$solID, starts_with("nrmse"), .data$decomp.rssd, .data$mape
+    )
+    outcome <- left_join(outcome, errors, "solID") %>% ungroup()
   } else {
-    outcome <- select(x, .data$solID, .data$rn, .data$cpa_total) %>%
-      filter(is.finite(.data$cpa_total)) %>%
-      tidyr::spread(key = .data$rn, value = .data$cpa_total)
+    if (cluster_by == "hyperparameters") {
+      outcome <- select(
+        x, .data$solID, contains(HYPS_NAMES),
+        contains(c("nrmse", "decomp.rssd", "mape"))
+      ) %>%
+        removenacols(all = FALSE)
+    }
   }
-
-  outcome <- removenacols(outcome, all = FALSE)
-  outcome <- select(outcome, any_of(c("solID", all_media)))
-  errors <- distinct(
-    x, .data$solID, .data$nrmse, .data$nrmse_test,
-    .data$nrmse_train, .data$decomp.rssd, .data$mape
-  )
-  outcome <- left_join(outcome, errors, "solID") %>% ungroup()
   return(outcome)
 }
 
 .min_max_norm <- function(x, min = 0, max = 1) {
   x <- x[is.finite(x)]
-  if (length(x) == 1) {
+  x <- x[!is.na(x)]
+  if (length(x) <= 1) {
     return(x)
-  } # return((max - min) / 2)
+  }
   a <- min(x, na.rm = TRUE)
   b <- max(x, na.rm = TRUE)
-  (max - min) * (x - a) / (b - a) + min
+  if (b - a != 0) {
+    return((max - min) * (x - a) / (b - a) + min)
+  } else {
+    return(x)
+  }
 }
 
 .clusters_df <- function(df, all_paid, balance = rep(1, 3), limit = 1, ts_validation = TRUE, ...) {
@@ -331,8 +372,9 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
   temp <- ifelse(dep_var_type == "conversion", "CPA", "ROAS")
   df_ci <- df_ci[complete.cases(df_ci), ]
   p <- ggplot(sim_collect, aes(x = .data$x_sim, y = .data$rn)) +
-    facet_wrap(~ .data$cluster_title) +
-    geom_density_ridges_gradient(scale = 3, rel_min_height = 0.01, size = 0.1) +
+    facet_wrap(~ .data$cluster_title, scales = "free_x") +
+    xlim(range(sim_collect$x_sim)) +
+    geom_density_ridges_gradient(scale = 3, rel_min_height = 0.01) +
     geom_text(
       data = df_ci,
       aes(x = .data$boot_mean, y = .data$rn, label = .data$boot_ci),
@@ -353,7 +395,8 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
         formatNum(sim_n, abbr = TRUE)
       )
     ) +
-    theme_lares(legend = "none")
+    theme_lares(background = "white", legend = "none") +
+    theme(axis.line.x = element_line())
   if (temp == "ROAS") {
     p <- p + geom_hline(yintercept = 1, alpha = 0.5, colour = "grey50", linetype = "dashed")
   }
@@ -382,7 +425,7 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
         round(100 * balance[1]), round(100 * balance[2]), round(100 * balance[3])
       )
     ) +
-    theme_lares()
+    theme_lares(background = "white", )
 }
 
 .plot_topsols_rois <- function(df, top_sols, all_media, limit = 1) {
@@ -391,10 +434,10 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
   top_sols %>%
     left_join(real_rois, by = c("solID" = "real_solID")) %>%
     mutate(label = sprintf("[%s.%s]\n%s", .data$cluster, .data$rank, .data$solID)) %>%
-    tidyr::gather("media", "roi", contains(all_media)) %>%
+    tidyr::gather("media", "perf", contains(all_media)) %>%
     filter(grepl("real_", .data$media)) %>%
     mutate(media = gsub("real_", "", .data$media)) %>%
-    ggplot(aes(x = reorder(.data$media, .data$roi), y = .data$roi)) +
+    ggplot(aes(x = reorder(.data$media, .data$perf), y = .data$perf)) +
     facet_grid(.data$label ~ .) +
     geom_col() +
     coord_flip() +
@@ -402,13 +445,12 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
       title = paste("Top Performing Models"),
       x = NULL, y = "Mean metric per media"
     ) +
-    theme_lares()
+    theme_lares(background = "white", )
 }
 
 .bootci <- function(samp, boot_n, seed = 1, ...) {
   set.seed(seed)
-
-  if (length(samp) > 1) {
+  if (length(samp[!is.na(samp)]) > 1) {
     samp_n <- length(samp)
     samp_mean <- mean(samp, na.rm = TRUE)
     boot_sample <- matrix(
@@ -428,6 +470,6 @@ errors_scores <- function(df, balance = rep(1, 3), ts_validation = TRUE, ...) {
 
     return(list(boot_means = boot_means, ci = ci, se = se))
   } else {
-    return(list(boot_means = samp, ci = c(NA, NA), se = NA))
+    return(list(boot_means = samp, ci = c(samp, samp), se = 0))
   }
 }
