@@ -129,7 +129,7 @@ robyn_converge <- function(OutputModels,
     mutate(id = as.integer(.data$cuts)) %>%
     mutate(cuts = factor(.data$cuts, levels = rev(levels(.data$cuts)))) %>%
     group_by(.data$error_type) %>%
-    mutate(value = lares::winsorize(.data$value, nrmse_win)) %>%
+    mutate(value = lares::winsorize(.data$value, nrmse_win), na.rm = TRUE) %>%
     ggplot(aes(x = .data$value, y = .data$cuts, fill = -.data$id)) +
     ggridges::geom_density_ridges(
       scale = 2.5, col = "white", quantile_lines = TRUE, quantiles = 2, alpha = 0.7
@@ -137,7 +137,7 @@ robyn_converge <- function(OutputModels,
     facet_grid(. ~ .data$error_type, scales = "free") +
     scale_fill_distiller(palette = "GnBu") +
     guides(fill = "none") +
-    theme_lares() +
+    theme_lares(background = "white", ) +
     labs(
       x = "Objective functions", y = "Iterations [#]",
       title = "Objective convergence by iterations quantiles",
@@ -146,10 +146,10 @@ robyn_converge <- function(OutputModels,
     )
 
   moo_cloud_plot <- df %>%
-    mutate(nrmse = lares::winsorize(.data$nrmse, nrmse_win)) %>%
+    mutate(nrmse = lares::winsorize(.data$nrmse, nrmse_win), na.rm = TRUE) %>%
     ggplot(aes(
-    x = .data$nrmse, y = .data$decomp.rssd, colour = .data$ElapsedAccum
-  )) +
+      x = .data$nrmse, y = .data$decomp.rssd, colour = .data$ElapsedAccum
+    )) +
     scale_colour_gradient(low = "skyblue", high = "navyblue") +
     labs(
       title = ifelse(!calibrated, "Multi-objective evolutionary performance",
@@ -163,7 +163,7 @@ robyn_converge <- function(OutputModels,
       alpha = NULL,
       caption = paste(conv_msg, collapse = "\n")
     ) +
-    theme_lares()
+    theme_lares(background = "white", )
 
   if (calibrated) {
     moo_cloud_plot <- moo_cloud_plot +
